//
//  TemplateImageShader.metal
//  SLFlops
//
//  Created by Silverlabs on 26/07/21.
//  Copyright © 2021 Silverlabs. All rights reserved.
//

#include <metal_stdlib>
using namespace metal;

struct VertexIn {
    float4 position [[ attribute(0) ]];
    float4 color [[ attribute(1) ]];
    float2 textureCoordinates [[ attribute(2) ]];
};

struct VertexOut {
    float4 position [[ position ]];
    float4 color;
    float2 textureCoordinates;
};

vertex VertexOut vertex_shader(const VertexIn vertexIn [[ stage_in ]]) {
    
    VertexOut vertexOut;
    vertexOut.position = vertexIn.position;
    vertexOut.color = vertexIn.color;
    vertexOut.textureCoordinates = vertexIn.textureCoordinates;
    return vertexOut;
}

fragment half4 textured_fragment(VertexOut vertexIn [[ stage_in ]],
                                 sampler sampler2d [[ sampler(0) ]],
                                 texture2d<float> texture [[ texture(0) ]] ) {
    float4 color = texture.sample(sampler2d, vertexIn.textureCoordinates);
    return half4(color.r, color.g, color.b, 1);
}

// MARK: - Masking Shader
fragment float4 textured_mask_fragment(VertexOut vertexIn [[stage_in]],
                                       texture2d<float> srcTexture [[texture(0)]],
                                       texture2d<float> maskTexture [[texture(1)]],
                                       sampler sampler2d [[sampler(0)]],
                                       constant int &maskComponent [[ buffer(0) ]]) {
    float4 srcColor = srcTexture.sample(sampler2d, vertexIn.textureCoordinates);
    
    float4 maskColor = maskTexture.sample(sampler2d, vertexIn.textureCoordinates);
    
    if (is_null_texture(maskTexture)) {
        return srcColor;
    }
    
    float4 maskedColor;
    if (maskComponent == 3) { // means no mask channel
        maskedColor = srcColor;
    } else {
        maskedColor = srcColor * maskColor[maskComponent];
    }
    return maskedColor;
}

// MARK: - Do not delete / May be used later
//fragment float4 textured_mask_fragment_multiple(VertexOut vertexIn [[stage_in]],
//                                                array<texture2d<float>, 20> srcTextures [[ texture(2)]],
//                                                texture2d<float> bgTexture [[texture(0)]],
//                                                texture2d<float> maskTexture [[texture(1)]],
//                                                sampler sampler2d [[sampler(0)]],
//                                                constant int &maskComponent [[ buffer(0) ]]) {
//    float4 maskColor = maskTexture.sample(sampler2d, vertexIn.textureCoordinates);
//    float4 srcColor = srcTextures[0].sample(sampler2d, vertexIn.textureCoordinates);
//
//    for (int i = 1; i < int(srcTextures.size()); i++) {
//        if (is_null_texture(srcTextures[i])) {
//            break;
//        }
//        float4 color = srcTextures[i].sample(sampler2d, vertexIn.textureCoordinates);
//        srcColor = srcColor * (1 - color.a) + color;
//    }
//    float2 bgCoordinates = float2(vertexIn.textureCoordinates.x * 0.5, vertexIn.textureCoordinates.y);
//    float2 fgCoordinates = float2(vertexIn.textureCoordinates.x * 0.5 + 0.5, vertexIn.textureCoordinates.y);
//
//    float4 bgColor = bgTexture.sample(sampler2d, bgCoordinates);
//    float4 fgColor = bgTexture.sample(sampler2d, fgCoordinates);
//
//    float4 maskedColor = srcColor * maskColor[maskComponent];
//
//    float4 bgAndSrc = bgColor * (1.0 - maskedColor.a) + maskedColor; // alpha blending bgImage and masked images
//    return 1.0 - (1.0 - bgAndSrc) * (1.0 - fgColor); // scrreen blend fg and mixed image
//}
